/**
 *@name routeMain.js: GUI handling
 * @version 1.3.13 for GoogleMaps V3
 * @copyright (c) 2010 B.Ostermann
 * @author Bernd Ostermann
 * @contact ostermann.bernd@arcor.de
 */
//global variables
var appVersion = "1.3.13";         //version of application RouteCngLpgFinder
var language;                   //English: "en" or German: "de"; set in html file
var xmlDetails = true;            //process Detail tags in XML files
var XLPG = 0;                     //gastype LPG
var XCNG = 1;                     //gastype CNG
var defaultGasType = XLPG;
var selectedGasType;            //actual selected gas type
//routeMessage_DE.js, routeMessage_EN.js: class Countries() with names of CNG- and LPG countries
var gasXmlObjects = null;         //actual XML objects of selected gas type, defined in createCountrySelectBox
var defaultLpgCountries = new Array(); //default Lpg countries
defaultLpgCountries["AT"] = true;
defaultLpgCountries["CH"] = true;
defaultLpgCountries["DE"] = true;

var defaultCngCountries = new Array();    //default Cng countries
defaultCngCountries["AT"] = true;
defaultCngCountries["CH"] = true;
defaultCngCountries["DE"] = true;
var selGasCountries;        //indices of selected LPG or CNG countries

var defaultMaxDistance = 5.0; //default maximum distance to route
var minMaxDistance = 1.0;      //minimum distance to route
var maxMaxDistance = 40.0;    //maximum distance to route
var defaultCruiseRange = 400.0; //cruising range with a full tank in kms
var minCruiseRange = 50.0;        //in kms
var maxCruiseRange = 1000.0;      //in kms
var actCruiseRange = defaultCruiseRange;
var startAdr = "4056 Basel, Hebelstr. 11, CH";
var endAdr = "Kufstein; München; 50.0524207, 8.5695232";
var map;        //the actual mapp
var mapCenter;  //the center of the map before a CNG station in the side bar was zoomed in
var mapZoom;    //the zoom level before a station in the side bar was zoomed in
var zoomState = null; //!=null: clicking the map returns to mapCenter, mapZoom
var mapDragging = false;  /*true: the user is just dragging the map*/
var directionDisplay;
var directionsService;
var routeObj = null;  //{route, route distance}
var routeSections = null; //array of route sections covering the route
var _polyGeodesic = null; //geodesic line from start to end of route
var routePolyline = null; //the polyline of the route
var refuelStops = null;   //array of elements RefuelStop
var xmlParser = null;    //instance of geoxmls3.parser
var sharedInfoWindow;   //the InfoWindow object used for all InfoWindows
var geoCoder = null;      //instance of the geoCoder
var markerMgr = null;   //instance of the markermanager
var poiMarkers = new Array();    //array of CngMarkers
var ptrToPoiMarkers = new Array(); //index pointing to the array poiMarkers
var countryCodes = new Array();
/**
 * Invoked, when the page is competely loaded
 */
$(document).ready(function() {
    language = "de";
    if (typeof google === 'undefined') {
        //google.maps was not loaded; no internet connection
        document.getElementById('getRoute').disabled = true;
        document.getElementById('getStations').disabled = true;
        document.getElementById('typeCng').disabled = true;
        document.getElementById('typeLpg').disabled = true;
        console.log("CngLpgFinder:" + MSG_NO_INTERNET_CONNECTION);
        alert(MSG_NO_INTERNET_CONNECTION);
        window.stop();
    } else {
        initialize();
        $("#infolist").accordion({
            autoHeight: false
        });
    }
});
/**
 *Invoked after the html file was loaded.
 */
function initialize() {
    setUiDefaults(); //sets default values of UI elements
    document.getElementById("getRoute").disabled = false;
    document.getElementById("typeLpg").disabled = false;
    document.getElementById("typeCng").disabled = false;
    //set event listeners
    var el=document.getElementById("btnParameters");
    el.addEventListener("click", function(){showLayer("params");}, false);
    el=document.getElementById("btnRouteResults");
    el.addEventListener("click", function(){showLayer("routelist");}, false);
    el=document.getElementById("btnStationResults");
    el.addEventListener("click", function(){showLayer("stationlist");}, false);
    el=document.getElementById("btnMap");
    el.addEventListener("click", function(){showLayer("map");}, false);
    el=document.getElementById("getRoute");
    el.addEventListener("click", onClickRoute, false);
    el=document.getElementById("getStations");
    el.addEventListener("click", onClickGetStations, false);
    el=document.getElementById("help");
    el.addEventListener("click", function(){onClickHelp('de');}, false);
    el=document.getElementById("GasCountries");
    el.addEventListener("mouseout", onmouseoutGasCountries, false);
//create the country select box of the selected gas type, sets the
    //default selections and loads the selected countries
    onClickGasType();
    //loads the map
    var latlng = new google.maps.LatLng(51.31893, 9.49601); //Kassel, Germany
    var myOptions = {
        zoom: 7,
        center: latlng,
        mapTypeControl: true,
        mapTypeControlOptions: {
            style: google.maps.MapTypeControlStyle.DROPDOWN_MENU
        },
        zoomControl: true,
        panControl: true,
        draggable: true,
        disableDoubleClickZoom: true,
        scrollwheel: true,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
        scaleControl: true
    };
    map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);
    var trafficLayer = new google.maps.TrafficLayer();
    trafficLayer.setMap(map);
    //create a shared InfoWindow
    sharedInfoWindow = new google.maps.InfoWindow({
        maxWidth: 300,
        pixelOffset: new google.maps.Size(0, 2)
    });
    //event listeners
    google.maps.event.addListener(map, 'dblclick', onRightclickMap);
    google.maps.event.addListener(map, 'click', onClickMap);
    google.maps.event.addListener(map, 'drag', function() {
        //if mapDragging is true, mapDistanceControl is disabled, to reduce CPU load
        mapDragging = true;
    });
    google.maps.event.addListener(map, 'idle', function() {
        mapDragging = false;
    });
    //add MapDistanceControl; not for IE, it is to slow
    var browserName = navigator.appName;
    /*
     if (browserName !== "Microsoft Internet Explorer") {
     var distCtrl = new MapDistanceControl();
     map.controls[google.maps.ControlPosition.TOP_RIGHT].push(distCtrl);
     google.maps.event.addListener(map, 'mousemove', function(event) {
     if (!mapDragging && routePolyline) {
     var dist = Math.round(1000 * getProximity(routePolyline, event.latLng).distance) / 1000;
     document.getElementById('distPtr').value = dist + " km";
     } else {
     document.getElementById('distPtr').value = "disabled while dragging.";
     }
     });
     }
     */
    directionsService = new google.maps.DirectionsService();
    geoCoder = new google.maps.Geocoder();
    directionsDisplay = new google.maps.DirectionsRenderer();
    directionsDisplay.setMap(map);
    directionsDisplay.setPanel(document.getElementById("results"));
    markerMgr = new MarkerManager(map);
    google.maps.event.addListener(markerMgr, 'loaded', function() {
        //alert("MarkerManager loaded. ");
        //onClickRoute();
    });
}
/**
 * Sets default values of UI elements
 *
 */
function setUiDefaults(){
    document.getElementById('fromAddress').value = startAdr;
    document.getElementById('toAddress').value = endAdr;
    document.getElementById('maxdistance').value = defaultMaxDistance;
    document.getElementById('cruiserange').value = defaultCruiseRange;
}

var currentLayer = 'params';
/**
 * Hides the current layer and shows the layer with id=lyr 
 * and sets the current layer to lyr.
 * @param {type} lyr id of the layer to be shown
 */
function showLayer(lyr) {
    hideLayer(currentLayer);
    $('html, body,div').animate({scrollTop: $("#navbar").offset().top}, 'fast');
    //window.scroll(100, 100); // horizontal and vertical scroll targets
    document.getElementById(lyr).style.visibility = 'visible';
    currentLayer = lyr;
}

function hideLayer(lyr) {
    document.getElementById(lyr).style.visibility = 'hidden';
}

/**
 * Loads the station markers of the seleted countries.
 * <br>Use of gobal variables:
 * <br>cngFileNames: the names of CNG files
 * <br>poiMarkers loaded CNG markers
 * <br>markerNames names of loaded CNG markers
 * @param selCountries indices of selected countries
 */
function loadStationMarkers(selCountries) {
    markerMgr.clearMarkers();
    var n = selCountries.length;
    var j;
    var xmlObjects = new Array();
    for (var i = 0; i < n; i++) {
        j = selCountries[i];
        xmlObjects.push(gasXmlObjects[j]);
    }
    poiMarkers = new Array();
    poiMarkerNames = new Array();
    poiMarkerDescs = new Array();
    poiMarkerCCs = new Array();
    document.getElementById("loadedStations").innerHTML = "";
    //Add markers of the xml files
    if (!xmlParser) {
        xmlParser = new GeoXML3({
            //map: map,
            zoom: true,
            processStyles: true,
            processDetails: xmlDetails,
            createMarker: addGasMarker,
            singleInfoWindow: true,
            infoWindowOptions: {
                maxWidth: 300
            }
        });
    }
    var docSet = xmlParser.parse(xmlObjects); //docs of docSet relevant only, if option map
    var k = poiMarkers.length;
    document.getElementById("loadedStations").innerHTML = k;
    signalBusyState(false);
}
/**
 *Event routine of GeoXML3: invoked for each extracted placemark
 *<br>Adds the placemark to the global array poiMarkers
 *@param placemark the placemark of the POI
 *@param countryCode CC; used to determine the path of the station gas file; e.g. for CNG:
 *<br>./cngjs/CC-CNG.js
 *@return google.maps.Marker
 */
function addGasMarker(placemark, countryCode) {
    var marker = createGasMarker(placemark);
    var gasStationPoi = new GasStationPoi(marker, placemark.name, placemark.description, placemark.detailUrl, countryCode);
    // add the marker onClick listener, which opens an info window
    google.maps.event.addListener(marker, 'click', function() {
        onMarkerClicked(gasStationPoi);
    });
    poiMarkers.push(gasStationPoi);
    return marker;
}

/**
 * Invoked after the name of POI in the output <DIV> was clicked.
 * <br>The map is shown and zoomed in and entered at the selected marker;
 * an info window of the marker is opened.
 * <br>Clicking the map closes the info window returns to zoom level and center 
 * before the info window was opened.
 * @param j the number of the selected marker.
 */
function zoomAroundPoi(j) {
    showLayer('map');   //show the layer with id='map'
    //var gasStationPoi = poiMarkers[j];
    onMarkerClicked(poiMarkers[j]);
}
function onMarkerClicked(gasStationPoi){
    var marker = gasStationPoi.marker;
    sharedInfoWindow.close();
    if (!zoomState) {
        mapZoom = map.getZoom();
        mapCenter = map.getCenter();
        zoomState = 1;    //map is zoomed
    }
    var latLng = marker.getPosition();
    map.panTo(latLng);
    map.setZoom(15);
    var content = gasStationPoi.toHtmlString();
    sharedInfoWindow.setContent(content);
    sharedInfoWindow.open(map, marker);
}


/**
 *Creates a marker from a placemark.
 *@param placemark {point,name,description,style}, point={lat, lng}; style={icon, shadow}
 */
function createGasMarker(placemark) {
    var markerOptions = {
        //map:      null,
        position: new google.maps.LatLng(placemark.point.lat, placemark.point.lng),
        title: placemark.name,
        //zIndex:   Math.round(-placemark.point.lat * 100000),
        icon: placemark.style.icon,
        shadow: placemark.style.shadow
    };
    // Create the marker
    var marker = new google.maps.Marker(markerOptions);
    return marker;
}
/**
 *Invoked after the map was rightclicked.
 *<br>Starts the geo coding of the mouse position.
 *<br>The result will be handled by the call back function:
 *<br>action:
 *<br>A infoWindow is opened with the address of the point and the
 * geogr. latitude, 
 * @param event the click event
 * 
 */
function onRightclickMap(event) {
    sharedInfoWindow.close();
    var latLng = event.latLng;
    var lat = Math.round(latLng.lat() * 100000.0) / 100000.0;
    var lng = Math.round(latLng.lng() * 100000.0) / 100000.0;
    /*
     geoCoder.geocode({
     'latLng': latLng,
     'language': language
     */
    geoCoder.geocode({
        'location': latLng,
        'region': language
    }, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK) {
            if (results[0]) {
                var text = results[0].formatted_address +
                        '<br/>Geogr. latitude, longitude:<br/>' + lat + ', ' + lng;
                sharedInfoWindow.setContent(text);
                sharedInfoWindow.setPosition(latLng);
                sharedInfoWindow.open(map);
            } else {
                alert("No results found.");
            }
        } else {
            alert(getGeoCoderErrorMsg(status));
        }
    });
}
/**
 *Invoked after the map was clicked;
 *<br>Actions:
 *<br>close the shared InfoWindow
 *<br>If the map was zoomed in by clicking an entry of the sidebar or a marker, the map
 *is set to the old values of center and zoom level
 *@param mouseEvent the click event
 */

function onClickMap(mouseEvent) {
    sharedInfoWindow.close();
    if (zoomState) {
        map.setCenter(mapCenter);
        map.setZoom(mapZoom);
        zoomState = null;
    }
}
/**
 *Invoked after the button generatePoi was clicked.
 *<br>Action:
 *<br>Generates a page with POIs of CNG stations, which were found nearby a route or nearby
 * a refueling stop.
 * <br>If the KML button is checked, then a KML file is generated, if the
 * CSV button is checked a comma separated CSV file.
 * Global variables:
 * <br> poiMarkers loaded poiMarkers with all CNG stations
 * <br> ptrToPoiMarkers the CNG stations nearby the route as index array, whose
 * elements point to poiMarkers.
 */
function generatePois() {
    var buttons = document.getElementsByName("poifile");
    var checked = buttons[0].checked; //index 0: KML radio button
    var poi;
    if (checked) {
        poi = new PoiAsKml();
    } else {
        poi = new PoiAsCsv();
    }
    var win = window.open("", "GasStationPois", "width=1024,height=768,left=20,top=20,scrollbars=yes,location=yes,menubar=yes,status=yes,toolbar=yes,resizable=yes");
    var doc = win.document;
    var html = new Array();
    html.push('<html><head>');
    html.push('<base href="">');
    html.push('<meta http-equiv="content-type" content="text/xml; charset=utf-8"/>');
    html.push('</head>');
    html.push('<body><div id="routecngs"></div></body>');
    html.push('</html>');
    doc.write(html.join(' '));
    var txt = new Array();
    txt.push(poi.getHeadTags());
    var noPois = ptrToPoiMarkers.length;
    var gasStationPoi;
    var latLng;
    var j;
    var marker;
    for (var i = 0; i < noPois; i++) {
        j = ptrToPoiMarkers[i];
        gasStationPoi = poiMarkers[j];
        marker = gasStationPoi.marker;
        latLng = marker.getPosition();
        txt.push(poi.getPoiElement(gasStationPoi.name, gasStationPoi.desc, latLng));
    }
    txt.push(poi.getEndTags());
    var el = doc.getElementById("routecngs");
    el.innerHTML = txt.join('');
    var rv;
    try {
        rv = doc.execCommand("SaveAs", true, "Route-Station.txt");
    } catch (exc) {
        alert(MSG_SAVEAS_KML);
    }
    win.focus();
}
function printResults() {
    /*
     var divToPrint=document.getElementById('results');
     var newWin=window.open('','Results','width=1024,height=700,scrollbars=yes,location=yes,menubar=yes,status=yes,toolbar=yes,resizable=yes');
     newWin.document.open();
     //newWin.document.write('<html><body onload="window.print()">'+divToPrint.innerHTML+'</body></html>');
     newWin.document.write('<html><body>'+divToPrint.innerHTML+'</body></html>');
     newWin.document.close();
     newWin.focus();
     //newWin.print();
     setTimeout(function(){newWin.print();},200);
     */
    var divToPrint = document.getElementById('results');
    var win = window.open("", "_blank", "width=900,height=768,left=20,top=20,scrollbars=yes,location=yes,menubar=yes,status=yes,toolbar=yes,resizable=yes");
    var doc = win.document;
    doc.open();
    var html = new Array();
    html.push('<html><head>');
    html.push('<base href="">');
    html.push('<meta http-equiv="content-type" content="text/html;charset=utf-8"/>');
    html.push('</head>');
    html.push('<body><div id="printDiv"></div></body>');
    html.push('</html>');
    doc.write(html.join(' '));
    var el = doc.getElementById("printDiv");
    el.innerHTML = divToPrint.innerHTML;
    doc.close();
    win.focus();
    setTimeout(function() {
        win.print();
    }, 100);
}

function showXmlFile(xmlObj) {
    var win = window.open("", "_blank", "width=900,height=768,left=20,top=20,scrollbars=yes,location=yes,menubar=yes,status=yes,toolbar=yes,resizable=yes");
    var doc = win.document;
    doc.open();
    var html = new Array();
    html.push('<html><head>');
    html.push('<base href="">');
    html.push('<meta http-equiv="content-type" content="text/xml; charset=utf-8"/>');
    html.push('</head>');
    //html.push('<body><div id="printDiv"><xmp id="xmpDiv"></xmp></div></body>');
    html.push('<body><xmp id="xmpDiv"></xmp></body>');
    html.push('</html>');
    doc.write(html.join(' '));
    var el = doc.getElementById("xmpDiv");
    el.innerHTML = xmlObj.gasXml;
    doc.close();
    win.focus();
}

/**
 *Invoked after the button 'getStations'was clicked.
 *<br>Actions:
 *<br>Checks, if the fields 'maxdistance' and 'criuserange' contain valid
 * numbers in the valid ranges.
 * <br>If the numbers are correct, the CNG stations within a distance from the route
 * of 'maxdistance' kms are shown.
 * <br>checkbox 'refuelstops' is checked: only the CNG stations at refuiling centers
 * at distances ('cruiseRange' - 'maxdistance') are shown
 */
function onClickGetStations() {
    var maxDist = getValidNumber("maxdistance", minMaxDistance, maxMaxDistance);
    var cruiseRange;
    if (maxDist) {
        var atRefuelStops = document.getElementById("refuelstops").checked;
        if (atRefuelStops) {
            cruiseRange = getValidNumber("cruiserange", minCruiseRange, maxCruiseRange);
            if (cruiseRange) {
                if (maxDist >= cruiseRange / 3) {
                    maxDist = Math.round(cruiseRange / 3);
                    document.getElementById("maxdistance").value = maxDist;
                }
                //shows only POIs at cruising ranges with route distance <=maxDist
                signalBusyState(true);
                document.getElementById("generatePoi").disabled = true;
                //Wait some time to make the disabled button visible
                window.setTimeout(function() {
                    showPois(maxDist, cruiseRange);
                }, 20);
            }
        } else {
            //show all POIs within a distance from theroute <=maxDist
            signalBusyState(true);
            document.getElementById("generatePoi").disabled = true;
            //Wait some time to make the disabled button visible
            window.setTimeout(function() {
                showPois(maxDist, 0);
            }, 20);
        }
    }
}
/**
 *Invoked when initialized or after a radio button LPG/CNG was clicked.
 *<br>If another radio button is checked, then the gas type is changed.
 *<br>Actions:
 *<br>The XML objects of the new gas type and the default selections are set.
 *<br>The html select box with options is created.
 *<br>The default selections are set.
 *<br>The POIs of the selected countries are loaded.
 */
function onClickGasType() {
    var buttons = document.getElementsByName("gastype");
    var index;
    var defaultCountries;
    if (buttons[XLPG].checked) {
        index = XLPG;
        defaultCountries = defaultLpgCountries;
    } else {
        index = XCNG;
        defaultCountries = defaultCngCountries;
    }
    if (selectedGasType == null || selectedGasType != index) {
        selectedGasType = index;

        //create the country select box
        var gasCountries = createCountrySelectBox(index);
        //set selected gas countries in the selection box
        selGasCountries = setSelectedCountries(defaultCountries, gasCountries);
        //the timer makes the new cursor style visible immediately
        signalBusyState(true);
        window.setTimeout(function() {
            loadStationMarkers(selGasCountries);
        }, 0);
    }
}

function createCountrySelectBox(gasTypeIndex) {
    if (gasTypeIndex == XLPG) {
        gasXmlObjects = [new AT_LPG(), new BE_LPG(), new CH_LPG(), new CZ_LPG(), new DE_LPG(), new DK_LPG(),
            new ES_LPG(), new FR_LPG(), new IT_LPG(), new LU_LPG(), new NL_LPG(), new NO_LPG(), new PL_LPG(),
            new PT_LPG(), new SE_LPG(), new UK_LPG()];
    } else {
        gasXmlObjects = [new AT_CNG(), new BE_CNG(), new CH_CNG(), new CZ_CNG(), new DE_CNG(),
            new FR_CNG(), new IT_CNG(), new LU_CNG(), new NL_CNG(), new NO_CNG(), new PL_CNG(), new SE_CNG()];
    }
    var nCountries = gasXmlObjects.length;
    var gasCountries = new Array();
    var countryNames = new Countries();  //all country names
    var countryCode;
    var name;
    //var html = '<select id="GasCountries" onmouseout="onmouseoutCngCountries(event)" MULTIPLE size="4" title="' + TITLE_COUNTRY_SELECT + '">';
    var html = "";
    for (var i = 0; i < nCountries; i++) {
        countryCode = gasXmlObjects[i].countryCode;
        name = countryNames.getName(countryCode);
        gasCountries.push({
            name: name,
            countryCode: countryCode
        });
        html += '<option value="' + i + '">' + name + '</option>';
    }
    //html += '</select>';
    var results = document.getElementById("GasCountries");
    results.innerHTML = html;
    return gasCountries;
}
function setSelectedCountries(defaultCountries, gasCountries) {
    var selectedCountries = new Array();
    var countryCode;
    for (var i = 0; i < gasCountries.length; i++) {
        countryCode = gasCountries[i].countryCode;
        if (defaultCountries.hasOwnProperty(countryCode)) {
            selectedCountries.push(i);
        }
    }
    //select countries of the country select box
    var j;
    var countries = document.getElementById("GasCountries");
    for (i = 0; i < selectedCountries.length; i++) {
        j = selectedCountries[i];
        countries.options[j].selected = true;
    }
    return selectedCountries;
}
/**
 * Invoked, after the mouse has left the Select box 'CngCountries'.
 * <br> event.relatedTarget points to the html element at the actual
 * position of the mouse cursor. If its name is not equal to SELECT and OPTION,
 * then the mosue has left the select box.
 * <br>action:
 * <br>If the selections were modified, the corresponding country cng files are loaded.
 * @param e event object
 */
function onmouseoutGasCountries(e) {
    var event = e;
    //window.event for IE
    if (!event)
        event = window.event;
    var relTarg = event.relatedTarget || event.toElement;
    if (relTarg) {
        var nodeName = relTarg.nodeName;
        //alert("target node name= " + nodeName);
        if (nodeName != "SELECT" && nodeName != "OPTION") {
            //alert("Außerhalb: " + nodeName);
            var countries = document.getElementById("GasCountries");
            var newSelected = new Array();
            var n = countries.options.length;
            var j = 0;
            //get the actual selected options
            for (var i = 0; i < n; i++) {
                if (countries.options[i].selected) {
                    newSelected[j] = i;
                    j++;
                }
            }
            n = newSelected.length;
            if (n < 1) {
                //no option selected; select old selections
                for (i = 0; i < selGasCountries.length; i++) {
                    j = selGasCountries[i];
                    countries.options[j].selected = true;
                }
                alert(NO_CNGCOUNTRY_SELECTED);
                return;
            }
            var bEqual = true;
            //determine, if selections are modified
            if (n == selGasCountries.length) {
                for (i = 0; i < n; i++) {
                    if (newSelected[i] != selGasCountries[i]) {
                        bEqual = false;
                        break;
                    }
                }
            } else {
                bEqual = false;
            }
            if (!bEqual) {
                //load the modified selections
                selGasCountries = newSelected;
                signalBusyState(true);
                //the timer makes the cursor change appearing immediately
                //loadCngMarkers(selCngCountries);
                window.setTimeout(function() {
                    loadStationMarkers(selGasCountries);
                }, 10);
            }
        }
    }
}
/**
 *Invoked after the help button was clicked
 *<br>Opens a window with the help document
 *@param language de: german version, en: english version
 */
function onClickHelp(language) {
    var file = "./help/help-" + language + ".html";
    var win = window.open(file, "Help", "width=800, height=400,scrollbars=yes,location=yes,menubar=yes,status=yes,toolbar=yes,resizable=yes");
    win.focus();
}
/**
 * Invoked, after the button 'getRoute' was clicked.
 * <br>parameters:
 * <br> Element id=avoidHighways: true without highways, false: with highways
 * <br> Element id=fromAddress: the start address
 * <br> Element id=toAddress: end addresses of routes
 * <br> gobal variable language= en|de
 * <br>action: get the route in travelmode Driving and 
 * show the result list of the route.
 */
function onClickRoute() {
    document.getElementById("generatePoi").disabled = true;
    var results = document.getElementById("results");
    results.innerHTML = "";
    results = document.getElementById("station_results");
    results.innerHTML = "";
    //var messageDiv = document.getElementById("message");
    //messageDiv.innerHTML = BROWSER_RECOMMENDATION;
    //var tm=document.getElementById("travelMode").value;
    var b = document.getElementById("avoidHighways").checked;
    var from = document.getElementById("fromAddress").value;
    if (from.length < 1) {
        alert(FROM_ADDRESS_EMPTY);
        signalBusyState(false);
        return;
    }
    var to = document.getElementById("toAddress").value;
    var wayPoints = checkToAddress(to);
    if (wayPoints.length > 0) {
        signalBusyState(true);
        to = wayPoints.pop().location;
        getRoute(from, to, wayPoints, language, google.maps.DirectionsTravelMode.DRIVING, b);
    } else {
        alert(TO_ADDRESS_EMPTY);
        signalBusyState(false);
    }
}
/**
 *Checks the To_Address field and extracts the single addresses.
 *<br>to_adress:= [address1; adress2;      ;] destination adress
 *@param input string of the To_Address.
 *@return array if waypoints; the array may be empty
 */
function checkToAddress(input) {
    var toAddress = input.trimSpec(); //remove leading and trailing blanks
    var length = toAddress.length;
    var wayPoints = new Array();
    if (length > 0) {
        var wayPoint;
        var address;
        var iStart = 0;
        var iPos;
        while (iStart < length) {
            iPos = toAddress.indexOf(";", iStart);
            if (iPos < 0) {
                //end of input, no semicolon found
                address = toAddress.substring(iStart, length).trimSpec();
                if (address.length > 0) {
                    wayPoint = {
                        location: address,
                        stopover: false
                    };
                    wayPoints.push(wayPoint);
                }
                break;
            }
            if (iPos > iStart) {
                address = toAddress.substring(iStart, iPos).trimSpec();
                if (address.length > 0) {
                    wayPoint = {
                        location: address,
                        stopover: true
                    };
                    wayPoints.push(wayPoint);
                }
            }
            iStart = iPos + 1;
        }
    }
    return wayPoints;
}
/**
 *Checks, if the field 'elementId' contains a valid number in the range of
 *'minVal', 'maxVal'.
 *@param elementId id of html field.
 *@param minVal minimum value
 *@param maxVal maximum values
 *@return val!=null: the correct value
 *<br> val=null: the vlaue is not valid
 */
function getValidNumber(elementId, minVal, maxVal) {
    var val = parseFloat(document.getElementById(elementId).value);
    if (!isNaN(val)) {
        if (val < minVal || val > maxVal) {
            alert("The value of " + elementId + " must be in the intervall [" + minVal +
                    " , " + maxVal + "]");
            val = null;
        }
    } else {
        alert("The value of " + elementId + " is no valid number.");
        val = null;
    }
    return val;
}
/**
 * Signals the application being busy by disable/enable the buttons
 * with ids "getStations" and "getRoute", and sets the mouse cursor to
 * the wait or defualt style.
 * <br>Button getStationsis enabled onl, if a route exists.
 * @param busy true: app is busy; false: app is not busy
 */
function signalBusyState(busy) {
    document.getElementById("getRoute").disabled = busy;
    var cursor;
    if (busy) {
        cursor = 'wait';
    } else {
        cursor = 'default';
        if (routeObj !== null) {
            document.getElementById("getStations").disabled = false;
        }
    }
    document.body.style.cursor = cursor;
}
function secondsToTime(seconds) {
    var date = new Date(seconds * 1000);
    var hh = date.getUTCHours();
    var mm = date.getUTCMinutes();
    var ss = date.getSeconds();
// These lines ensure you have two-digits
    if (hh < 10) {
        hh = "0" + hh;
    }
    if (mm < 10) {
        mm = "0" + mm;
    }
    if (ss < 10) {
        ss = "0" + ss;
    }
    return hh + ":" + mm + ":" + ss;
}
/**
 * Invoked, after the checkbox 'refuelstops' was clicked.
 * <br>Action:
 * <br>If the checkbox is checked, the field 'criuserange' is enabled,
 * otherwise disabled.
 */
/*
 function onClickRefuelStops(){
 if (document.getElementById("refuelstops").checked){
 document.getElementById("cruiserange").disabled=false;
 }else{
 document.getElementById("cruiserange").disabled=true;
 }
 }
 */
/**
 * Starts the query for a route from 'fromAddress' to 'toAddress'.
 * <br>Before the query is started,  former results were cleared.
 * <br>The event routines onGDirectionsLoad and handleErrors are invoked,
 * after the query is finished or an error occurred.
 * @param fromAddress address of the start point
 * @param toAddress address of the end point
 * @param wayPoints addresses of stopover
 * @param locale language of the resulting route description
 * @param trMode travelmode 'driving' or 'walking'
 * @param bAvoidHw =true: avoid highways; =false: use highways too
 */
function getRoute(fromAddress, toAddress, wayPoints, locale, trMode, bAvoidHw) {
    sharedInfoWindow.close();
    markerMgr.clearMarkers();
    if (routePolyline) {
        routePolyline.setMap(null); //removes polyline from the map
        routePolyline = null;
        if (_polyGeodesic) {
            _polyGeodesic.setMap(null);
            _polyGeodesic = null;
        }
    }
    if (routeSections) {
        routeSections.clearBoxes();
        routeSections = null;
    }
    if (refuelStops) {
        refuelStops.clear();
        refuelStops = null;
    }
    var request = {
        origin: fromAddress,
        destination: toAddress,
        travelMode: trMode,
        avoidHighways: bAvoidHw,
        region: locale,
        waypoints: wayPoints,
        optimizeWaypoints: true //requires waypoints with stopover=true
    };
    directionsService.route(request, function(response, status) {
        if (status == google.maps.DirectionsStatus.OK) {
            directionsDisplay.setDirections(response);
            var route = response.routes[0];
            var routePolylineObj = createRoutePoly(route);
            routePolyline = routePolylineObj.polyline;
            var durationSecs = routePolylineObj.polylineDuration;
            var resultsOverviewDiv = document.getElementById("results_overview");
            var txt = "<b>" + LBL_FROM + "</b>" + fromAddress
                    + ", <b>" + LBL_TO + "</b>" + toAddress
                    + "<br/><b>" + LBL_DISTANCE + "</b>" + routePolylineObj.polylineLength + " km"
                    + ", <b>" + LBL_DURATION + "</b>" + secondsToTime(durationSecs);
            resultsOverviewDiv.innerHTML = txt;
            routeObj = {
                route: route,
                bounds: route.bounds,
                routePolyline: routePolyline,
                routeLength: routePolylineObj.polylineLength
            };
            //create a geodesic line from start to end point of the route
            var polyPath = routePolyline.getPath();
            var path = new Array();
            path.push(polyPath.getAt(0));
            path.push(polyPath.getAt(polyPath.length - 1));
            var polyOpts = {
                map: map,
                path: path,
                geodesic: true,
                strokeColor: "#ff0000",
                strokeOpacity: 1,
                strokeWeight: 2
            };
            showLayer('routelist'); //show the layer result list
            _polyGeodesic = new google.maps.Polyline(polyOpts);
            //map.panToBounds(route.bounds);
            map.fitBounds(route.bounds);
            mapZoom = map.getZoom();
            mapCenter = map.getCenter();
        } else {
            handleDirectionError(status);
        }
        signalBusyState(false);
    });
}


/**
 *Invoked after query of a route was finished with errors.
 *<br>Action:
 *<br>Evalutes the status code and display an error message.
 *@param status status of the route query
 */
function handleDirectionError(status) {
    alert(getDirectionErrorMsg(status));
    signalBusyState(false);
}

/**
 *Creates the polyline of a route.
 *@param route the route object
 *@return {polyline,route distance in kms}
 */
function createRoutePoly(route) {
    var legs = route.legs;
    var path = new Array();
    var steps, leg;
    var polylineLength = 0.0;
    var polylineDuration = 0;
    var dur;
    var lng;
    for (var i = 0; i < legs.length; i++) {
        leg = legs[i];
        lng = leg.distance.value;
        if (lng) {
            polylineLength += lng;
        }
        dur = leg.duration.value;
        if (dur) {
            polylineDuration += dur;
        }
        steps = leg.steps;
        for (var s = 0; s < steps.length; s++) {
            path = path.concat(steps[s].path);
        }
    }
    var polyOpts = {
        map: map,
        path: path,
        strokeColor: "#0000ff",
        strokeOpacity: 0.9,
        strokeWeight: 2
    };
    var polyline = new google.maps.Polyline(polyOpts);
    //polyline.setVisible(true);
    return {
        polyline: polyline,
        polylineLength: polylineLength / 1000.0,
        polylineDuration: polylineDuration
    };
}

/**
 * Clears existing markers and polygons and shows the POIs with a distance from the route less than
 * 'maxDistance' on the map.
 * <br>CriuseRange!=null: shows only POIs in the environment of a refuiling centers,
 * which are distributed over the route in a distance of ('cruiseRange' - 'maxDistance').
 * @param maxDistance the maximum distance of a marker to the route in kms
 * @param cruiseRange distance between 2 tank stops in kms
 */
function showPois(maxDistance, cruiseRange) {
    //var divMessages = new DivMessageTexts();
    sharedInfoWindow.close();
    markerMgr.clearMarkers();
    if (zoomState) {
        map.setCenter(mapCenter);
        map.setZoom(mapZoom);
        zoomState = null;
    }
    if (routeSections) {
        routeSections.clearBoxes();
        routeSections = null;
    }
    if (refuelStops) {
        refuelStops.clear();
        refuelStops = null;
    }
    var filterObj;
    if (cruiseRange > 0) {
        //gets all refueling stops at route distances n* (cruiseRange - maxDistance)
        refuelStops = new RefuelStops(cruiseRange, sharedInfoWindow,
                map, routeObj, maxDistance);
        filterObj = refuelStops;
    } else {
        routeSections = new RouteSections(map, routeObj, maxDistance);
        filterObj = routeSections;
    }
    //filters the poi markers with a distance from the route <=maxDistance
    ptrToPoiMarkers = new Array();
    var selectedMarkers = new Array();
    /*
     var messageDiv = document.getElementById("message");
     var now = new Date();
     var startMillis = now.getTime();
     var curr_hour = now.getHours();
     var curr_min = now.getMinutes();
     var curr_sec = now.getSeconds();
     var curr_msec = now.getMilliseconds();
     var html = divMessages.searchStart + curr_hour + ":" + curr_min + ":"
     + curr_sec + ":" + curr_msec;
     messageDiv.innerHTML = html;
     */
    var gasStationPoi;
    var marker;
    var j = 0;
    var k;
    var latLng;
    var noPoiMarkers = poiMarkers.length;
    for (k = 0; k < noPoiMarkers; k++) {
        gasStationPoi = poiMarkers[k];
        marker = gasStationPoi.marker;
        latLng = marker.getPosition();
        var distances = filterObj.filterPoi(latLng);
        if (distances) {
            selectedMarkers[j] = marker;
            gasStationPoi.distanceFromRoute = distances.distanceFromRoute;  //airline distance from the route
            gasStationPoi.distanceOnRoute = distances.distanceOnRoute; //distance on route from route start
            ptrToPoiMarkers[j] = k;
            j++;
        }
    }
    if (selectedMarkers.length > 0) {
        ptrToPoiMarkers.sort(compareByDistOnRoute);
        //without max zoom level: max zoom level of the markers= max zoon level of of map
        markerMgr.addMarkers(selectedMarkers, 4); //
        markerMgr.refresh();
    }
    /*
     now = new Date();
     var dur = now.getTime() - startMillis;
     html = messageDiv.innerHTML;
     html += "<br>" + divMessages.searchEnd + dur + " ms</br>";
     messageDiv.innerHTML = html;
     */
    outputPois(ptrToPoiMarkers, maxDistance);
    document.getElementById("generatePoi").disabled = false;
    signalBusyState(false);
    showLayer('stationlist'); //show the layer station list
}
/**
 *Sorting the array indexMarker compares 2 elements by their distances
 * on the route from route start
 *@param ind1 index of element 1 with resperct to distOnRoute
 *@param ind2 index of element 2 with resperct to distOnRoute
 *@return -1, 0, +1 for distance/element1)-distance(element2) <0,=0,<0.
 */
function compareByDistOnRoute(ind1, ind2) {
    var gasStationPoi1 = poiMarkers[ind1];
    var gasStationPoi2 = poiMarkers[ind2];
    var dist1 = gasStationPoi1.distanceOnRoute;
    var dist2 = gasStationPoi2.distanceOnRoute;
    var d = dist1 - dist2;
    return ((d < 0.0) ? -1 : ((d > 0) ? 1 : 0));
}
/**
 * Writes the resulting POIs to the <DIV> element 'results' in the order
 * of the index array.
 * <br>The name of the POI is clickable. When clicked a window is popped up.
 * @param index index[i] points to the i. POI in array poiMarkers
 * @param maxDistance the maximum distance from the route in kms.
 */

function outputPois(index, maxDistance) {
    var noPois = index.length;
    var divResults = new DivResultsTexts(); //english or german text
    var html = "<b>" + divResults.title + "</b>" +
            "<br>" + divResults.maxDist + maxDistance + " km" +
            "<br>" + divResults.numberStations + noPois;
    var poiFirstLine;
    var gasStationPoi;
    var titleLine;
    var body;
    var j;
    for (var i = 0; i < noPois; i++) {
        j = index[i];
        gasStationPoi = poiMarkers[j];
        gasStationPoi.orderNumber = i + 1;
        titleLine = gasStationPoi.titleToHtmlString();
        poiFirstLine = '<p>' + ' <a href="javascript:zoomAroundPoi(' + j + ')">' + titleLine + "</a>";
        body = gasStationPoi.bodyToHtmlString() + "</p>";
        html += (poiFirstLine + body);
    }
    var results = document.getElementById("station_results");
    results.innerHTML = html;
}



