/* 
 * @name routeMessages_De.js: messages in German.
 * @version 1.3.8
 * @copyright (c) 2010 B.Ostermann
 * @author Bernd Ostermann
 * @contact ostermann.bernd@arcor.de
 */
/**
 * Gets the error message after a route query was issued.
 * <br>Status code of DirectionsService:
 *<br>http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsService
 *@param statusCode status code of Directions service
 **/

function getDirectionErrorMsg(statusCode){
    var rv;
    switch(statusCode){
        case google.maps.DirectionsStatus.NOT_FOUND:
            rv="Es konnte kein geographischer Ort fuer eine der angegebenen Adressen gefunden werden, weil die Adresse relativ neu oder inkorrekt ist.";
            break;
        case google.maps.DirectionsStatus.MAX_WAYPOINTS_EXCEEDED:
            rv="Zu viele Ziel-Adresse. Es sind maximal 9 Ziel-Adressen zulässig.";
            break;
        case google.maps.DirectionsStatus.INVALID_REQUEST:
            rv="Unzulässige Routen-Anfrage.";
            break;
        case google.maps.DirectionsStatus.OVER_QUERY_LIMIT:
            rv="Es wurden zu viele Routen-Anfragen in der letzten Zeit gestellt.";
            break;
        case google.maps.DirectionsStatus.REQUEST_DENIED:
            rv="Routen-Anfragen sind nicht erlaubt, weil das Anfrage-Limit zu häufig überschritten wurde.";
            break;
        case google.maps.DirectionsStatus.ZERO_RESULTS:
            rv="Die Route konnte nicht ermittelt werden, weil erforderlich Daten nicht vohanden sind.";
            break;
        default:
            rv="Ein unbekannter Fehler trat auf mit Fehler-Code " + statusCode;
    }
    return rv;
}

/**
 * Gets the error message of the google.maps.GeoCoder.
 * @param statusCode the status code returned by the GeoCoder
 */
function getGeoCoderErrorMsg(statusCode){
    var rv;
    switch(statusCode){
        case google.maps.GeocoderStatus.ERROR:
            rv=MSG_NO_INTERNET_CONNECTION;
            break;
        case google.maps.GeocoderStatus.INVALID_REQUEST:
            rv="Diese Geocoder Anfrage war nicht korrekt.";
            break;
        case google.maps.GeocoderStatus.OVER_QUERY_LIMIT:
            rv="Zuviele Anfragen an den GoogleMaps Geocoder in zu kurzer Zeit."+
            "\nWarnung: Bei weiterem Überschreiten des Anfrage-Limits kann der Zugriff auf den Geocoder blockiert werden.";
            break;
        case google.maps.GeocoderStatus.REQUEST_DENIED:
            rv="Es ist Ihnen nicht erlaubt den GoogleMaps Geocoder zu nutzen;" +
            "\nmöglicherweise haben Sie das Anfrage-Limit zu häufig überschritten.";
            break;
        case  google.maps.GeocoderStatus.ZERO_RESULTS:
            rv="Für diese Anfrage an den Geocoder wurde kein Ergebnis gefunden.";
            break;
        case google.maps.GeocoderStatus.UNKNOWN_ERROR:
            rv="Wegen eines Server-Fehlers konnte diese Anfrage an den Geocoder nicht bearbeitet werden." +
                "\nVersuchen Sie es später noch einmal.";
            break;
        default:
            rv="Ein unbekannter Fehler trat auf. Fehler-Code " + statusCode;
    }
    return rv;
}
/**
     * Texts for the message field <div id="message"
     */
 /*
function DivMessageTexts(){
    this.searchStart="Startzeit fuer Suche von Gas-Tankstellen= ";
    this.searchEnd="Zeit fuer Suche von Gas-Tankstellen= ";
}
*/
/**
     * Texts for the field <div  id="results" ..>
     */
function DivResultsTexts(){
    this.title="Gas-Tankstellen entlang Route";
    this.maxDist="Max. Abstand von Route: ";
    this.numberStations="Anzahl Gas-Tankstellen: ";
    this.distOnRoute="Distanz auf Route: ";
    this.distFromRoute="Distanz Luftlinie von Route: ";
    this.latLng="Geogr. Breite/Länge: ";
}

/**
 * Text of marker description
 */
function MarkerDesc(){
    this.refuelCenter="Tankstop bei: ";
}
/**
 * Countries with LPG and/or CNG refueling stations
 */
function Countries(){
    var countryNames=new Array();
    countryNames["AT"]="AT Österreich";
    countryNames["BE"]="BE Belgien";
    countryNames["CH"]="CH Schweiz";
    countryNames["CZ"]="CZ Tschechei";
    countryNames["DE"]="DE Deutschland";
    countryNames["DK"]="DK Dänemark";
    countryNames["ES"]="ES Spanien";
    countryNames["FR"]="FR Frankreich";
    countryNames["IT"]="IT Italien";
    countryNames["LU"]="LU Luxenburg";
    countryNames["NL"]="NL Niederlande";
    countryNames["NO"]="NO Norwegen";
    countryNames["PL"]="PL Polen";
    countryNames["PT"]="PT Portugal";
    countryNames["SE"]="SE Schweden";
    countryNames["UK"]="UK Großbritanien"

    /**
 * Gets the name of a country
 * @param countryCode country code of the country
 * @return name of the country
 */
    this.getName=function(countryCode){
        return countryNames[countryCode];
    };
}

//messages
var MSG_NO_INTERNET_CONNECTION="Keine Netzwerk-Verbindung. Aktivieren Sie diese.";
//var TITLE_COUNTRY_SELECT="Auswahl von Ländern mit Gas-Tankstellen; Mehrfachauswahl mit STRG+Klick.";
var NO_CNGCOUNTRY_SELECTED="Es muss mindestens ein Land ausgewählt werden.";
var MSG_SAVEAS_KML="Speichern Sie die Seite als Textdatei mit der Namenserweiterung .txt: \n" +
"Menu 'Datei', Menupunkt 'Seite speichern unter'.\n" +
"Benennen Sie die Text-Datei um in *.kml oder *.csv.";
//var BROWSER_RECOMMENDATION="Verwenden Sie Firefox, Chrome, Safari oder Opera. Internet Explorer ist zu langsam!";
var TO_ADDRESS_EMPTY="Die Nach-Adresse darf nicht leer sein.";
var FROM_ADDRESS_EMPTY="Die Von-Adresse darf nicht leer sein.";
//var XML_TIMEOUT="Das Lesen einer XML-Datei mit Gas-Tankstellen wurde wegen Zeitüberschreitung abgebrochen.\n" +"Möglicherweise besteht keine Verbindung zum Internet.";
var LBL_FROM="Von: ";
var LBL_TO="Nach: ";
var LBL_DISTANCE="Gesamtdistanz: ";
var LBL_DURATION="Gesamtdauer: ";
