/* 
 script:
 * @name routeMessages_En.js: classes messages in English.
 * @version 1.3.8
 * @copyright (c) 2010 B.Ostermann
 * @author Bernd Ostermann
 * @contact ostermann.bernd@arcor.de
 */

/**
 * Gets the error message after a route query was issued.
* <br>Status code of DirectionsService:
 *<br>http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsService
*@param statusCode return code of Direction service
 */
function getDirectionErrorMsg(statusCode){
    var rv;
     switch(statusCode){
        case google.maps.DirectionsStatus.NOT_FOUND:
            rv="A geographical location could not be found for one of the given addrresses;" +
                "\nthe address may be invalid or relatively new.";
            break;
        case google.maps.DirectionsStatus.MAX_WAYPOINTS_EXCEEDED:
            rv="Too many destination addresses; maximum of 9 addresses are allowed.";
            break;
        case google.maps.DirectionsStatus.INVALID_REQUEST:
            rv="Invalid direction request.";
            break;
        case google.maps.DirectionsStatus.OVER_QUERY_LIMIT:
            rv="The request limit per day was exceeded." +
                "\nWarning: if the request limit is exceede too often, further requests may be denied.";
            break;
        case google.maps.DirectionsStatus.REQUEST_DENIED:
            rv="Direction request was denied;" +
                "possibly you have exceeded the request limit too often.";
            break;
        case google.maps.DirectionsStatus.ZERO_RESULTS:
            rv="The route could not be determined; required route data are not available.";
            break;
        default:
            rv="An unknown error occured; error code: " + statusCode;
    }
    return rv;
}
/**
 * Gets the error message of the google.maps.GeoCoder.
 * @param statusCode the status code returned by the GeoCoder
 */
function getGeoCoderErrorMsg(statusCode){
    var rv;
    switch(statusCode){
        case google.maps.GeocoderStatus.ERROR:
            rv=MSG_NO_INTERNET_CONNECTION;
            break;
        case google.maps.GeocoderStatus.INVALID_REQUEST:
            rv="This GeocoderRequest was invalid.";
            break;
        case google.maps.GeocoderStatus.OVER_QUERY_LIMIT:
            rv="The request limit of GoogleMaps Geocoder was exceeded in too short a period of time." +
            "\nWarning: If you continue to exceed this limit, your access to the Geocoding API may be blocked.";
            break;
        case google.maps.GeocoderStatus.REQUEST_DENIED:
            rv="You are not allowed to use the GoogleMaps Geocoder;" +
            "may be the request limit of GoogleMaps Geocoder was exceeded too often.";
            break;
        case  google.maps.GeocoderStatus.ZERO_RESULTS:
            rv="No result was found for this GeocoderRequest.";
            break;
        case google.maps.GeocoderStatus.UNKNOWN_ERROR:
            rv="A geocoding request could not be processed due to a server error. \nThe request may succeed if you try again.";
            break;
        default:
            rv="An unknown error occurred. Error code " + statusCode;
    }
    return rv;
}
/**
     * Texts for the message field <div id="message" ..>
     */
/*
function DivMessageTexts(){
    this.searchStart="Start searching for Gas stations= ";
    this.searchEnd="Time for searching Gas stations= ";
}
*/

/**
     * Texts for the field <div  id="results" ..>
     */
function DivResultsTexts(){
    this.title="Gas stations along the route";
    this.maxDist="Maximum distance from route: ";
    this.numberStations="Number of gas stations: ";
    this.distOnRoute="Distance on route: ";
    this.distFromRoute="Airline distance from route: ";
    this.latLng="Geogr. latitude and longitude: ";
}

/**
 * Text of marker description
 */
function MarkerDesc(){
    this.refuelCenter="center of refueling stops at: ";
}

/**
 * Countries with LPG and/or CNG refueling stations
 */
/**
 * Countries with LPG and/or CNG refueling stations.
 */
function Countries(){
    var countryNames=new Array();
    countryNames["AT"]="AT Austria";
    countryNames["BE"]="BE Belgium";
    countryNames["CH"]="CH Switzerland";
    countryNames["CZ"]="CZ Czechia";
    countryNames["DE"]="DE Germany";
    countryNames["DK"]="DK Denmark";
    countryNames["ES"]="ES Spain";
    countryNames["FR"]="FR France";
    countryNames["IT"]="IT Italy";
    countryNames["LU"]="LU Luxenbourg";
    countryNames["NL"]="NL Netherlands";
    countryNames["NO"]="NO Norway";
    countryNames["PL"]="PL Poland";
    countryNames["PT"]="PT Portugal";
    countryNames["SE"]="SE Sweden";
    countryNames["UK"]="UK Great Britain";


    /**
 * Gets the name of a country
 * @param countryCode country code of the country
 * @return name of the country
 */
    this.getName=function(countryCode){
        return countryNames[countryCode];
    };
}

//messages
var MSG_NO_INTERNET_CONNECTION="No network connection; must be activated.";
//var TITLE_COUNTRY_SELECT="Select countries with gas stations; multiple selection by CTRL+click.";
var NO_CNGCOUNTRY_SELECTED="At least one country must be selected.";
var MSG_SAVEAS_KML="Save this page as a text file with extension .txt:\n" +
"menu File, menu item SaveAs\n" +
"Rename the file to *.kml or *.csv";
//var BROWSER_RECOMMENDATION="Use Firefox, Chrome, Safari oder Opera. Internet Explorer is too slow!";
var TO_ADDRESS_EMPTY="To address must not be empty.";
var FROM_ADDRESS_EMPTY="From address must not be empty.";
//var XML_TIMEOUT="Parsing a XML file with Gas stations failed due to timeout.\n" + "Possibly the web connection is down";
var LBL_FROM="From: ";
var LBL_TO="To: ";
var LBL_DISTANCE="Total distance: ";
var LBL_DURATION="Total duration: ";